
# q-expansion of newform 6912.2.a.y, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6912
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [-3, 1], [-1, -3], [3, 2], [0, 0], [0, 0], [0, 0], [0, 0], [0, 2], [-5, 3], [0, 0], [0, 0], [0, 0], [0, 0], [-3, -5], [0, 8], [0, 0], [0, 0], [0, 0], [7, -6], [10, 0], [15, -2], [0, 0], [-1, -12], [-3, 7], [14, 0], [-15, -4], [0, 0], [0, 0], [-11, 3], [-3, -8], [0, 0], [0, 0], [-21, 1], [-1, 15], [0, 0], [0, 0], [0, 0], [-15, -7], [-21, 4], [0, 0], [0, 0], [-13, 6], [-21, -5], [-7, 15], [0, 0], [-26, 0], [0, -20], [0, 0], [0, 0], [0, 0], [-10, 0], [0, -4], [0, 0], [0, 0], [0, -22], [-11, -15], [0, 0], [0, 0], [0, 0], [0, 10], [0, 0], [0, 0], [-17, -6], [-15, 11], [0, 0], [-22, 0], [21, -10], [0, 0], [0, 0], [0, 0], [19, 3], [0, 0], [0, 0], [0, 0], [-21, -11], [0, 0], [0, 0], [-5, -24], [0, 28], [0, 0], [0, 0], [-7, 24], [17, 15], [0, 20], [0, 0], [19, 12], [-33, -7], [13, 21], [15, -14], [0, 0], [2, 0], [-33, 8], [0, 0], [0, 0], [39, -1], [0, 0], [0, 0], [0, 0], [0, 0], [39, 5], [33, 10], [0, 0], [0, 0], [38, 0], [-15, -16], [0, 0], [0, 0], [1, 30], [22, 0], [0, 0], [0, 0], [0, 0], [-25, -3], [0, 0], [0, 0], [0, 0], [-15, 17], [-21, 16], [0, 0], [17, 24], [0, -2], [0, -4], [0, 0], [-33, -13], [0, 0], [0, 0], [-1, 33], [0, 0], [0, 0], [0, 0], [-5, -33], [0, 0], [0, 0], [-13, -30], [0, -14], [0, 0], [-15, -19], [0, 0], [0, 0], [0, 34], [23, -21], [0, -40], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [-25, -21], [0, 0], [-29, 12], [-33, 17], [-21, -20], [0, 0], [-31, -3], [3, -22], [0, 0], [0, 0], [-29, -15], [0, 0], [-25, 24], [0, -26], [39, -16], [0, 0], [0, 0], [-26, 0], [-5, 39], [0, 0], [0, 0], [-3, -23], [-7, -39], [0, 0], [19, -33], [-57, -2], [0, 0], [0, 0], [0, 0], [51, -11], [0, 0], [0, 0], [-23, 30], [0, 0], [-34, 0], [-33, 20], [0, 0], [57, -7], [0, -32], [0, 0], [5, 42], [0, 0], [0, 0], [0, 0], [39, -19], [-35, 3], [0, 0], [35, 6], [-51, 14], [0, 50], [70, 0], [-57, 10], [0, 0], [0, 0], [11, 42], [0, 46], [17, -39], [39, 20], [0, 0], [-35, 12], [58, 0], [0, 0], [0, 0], [0, -10], [0, 0], [14, 0], [0, 0], [-74, 0], [15, -26], [0, 0], [0, 0], [0, 0], [62, 0], [0, -16], [0, 0], [0, 0], [22, 0], [0, 0], [0, 0], [0, 0], [50, 0], [0, -50], [21, 26], [0, 0], [0, -20], [0, 0], [23, -39], [0, 0], [0, 0], [0, 0], [-29, -33], [-3, 28], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [-35, -24], [-33, -25], [69, 4], [0, 0], [0, 0], [69, -5], [-11, 48], [37, 21], [51, -20], [0, 0], [0, 0], [0, 0], [0, 0], [0, -58], [0, 0], [0, 0], [0, 46], [0, 0], [0, 0], [37, -24], [-5, -51], [41, -12], [-7, 51], [-69, -10], [0, 0], [25, -42], [0, -56], [0, 0], [41, 15], [0, 0], [0, 0], [0, 0], [0, 0], [-43, 6], [-75, 1], [-46, 0], [0, 0], [0, -34], [-75, 4], [0, 0], [-33, -28], [0, 0], [0, 62], [-35, -33], [-75, 7], [-69, 14], [0, 0], [-14, 0], [-15, -31], [43, -15], [-75, -8], [0, 0], [29, 42], [0, -52], [0, 0], [0, 0], [0, 0], [0, 0], [-21, 31], [0, 0], [0, 0], [0, 0], [70, 0], [69, 16], [0, 0], [26, 0], [0, 0], [0, 0], [31, -42], [57, 23], [43, 21], [0, 0], [-25, -48], [0, 0], [0, 0], [0, 0], [-75, 13], [0, 0], [0, -10], [-35, 39], [0, -56], [0, 0], [75, 14], [0, 0], [0, 0], [-41, 30], [-11, 57], [0, 0], [0, 0], [69, 19], [62, 0], [57, -25], [0, -68], [0, 0], [0, 0], [0, 0], [0, 58], [0, 0], [-29, 48], [0, -46], [37, 39], [0, 0], [0, 0], [0, 0], [-33, 32], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [21, -34], [0, 0], [46, 0], [0, -70], [47, -21], [-11, -60], [87, -2], [0, 0], [0, 0], [51, -29], [-49, 15], [0, 0], [0, 28], [0, 0], [47, 24], [0, 0], [-98, 0], [-87, -7], [0, 0], [2, 0], [0, 0], [0, 0], [0, 0], [10, 0], [0, 0], [0, 0], [0, 0], [17, 60], [0, 70], [0, 32], [0, 0], [0, 0], [-86, 0], [-74, 0], [0, 0], [0, 0], [-3, 37], [0, 0], [0, 0], [22, 0], [0, 0], [51, 31], [-25, -57], [0, 0], [0, 0], [0, 0], [-39, 34], [53, 6], [69, 25], [75, -22], [0, 0], [-41, -42], [-87, -13], [0, 0], [98, 0], [0, 0], [0, 0], [0, 38], [0, 0], [0, 0], [-69, 26], [7, 66], [39, 35], [15, -38], [0, 0], [0, 0], [0, 0]]
